package com.gic.haoban.manage.service.service.impl;


import static org.slf4j.LoggerFactory.getLogger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.haoban.common.utils.DingUtils;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedDTO;
import com.gic.haoban.manage.api.dto.MemberUnionidRelatedDTO;
import com.gic.haoban.manage.api.dto.SecretSettingDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.WxUserAddLogQDTO;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.MemberUnionidRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanExternalClerkRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanRepairExternalUserMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStaffClerkRelationMapper;
import com.gic.haoban.manage.service.entity.MemberUnionidRelated;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.entity.TabHaobanRepairExternalUser;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.service.ExternalClerkRelatedService;
import com.gic.haoban.manage.service.service.SecretSettingService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.WxUserAddLogService;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberApiService;
import com.gic.member.api.service.MemberService;
import com.gic.mq.sdk.util.ProviderLocalTagUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.search.engine.api.dto.ESResponseQuerySingle;
import com.gic.search.engine.api.dto.dynamic.DynamicSearchDTO;
import com.gic.search.engine.api.service.dynamic.ESDataDynamicOperationApiService;
import com.gic.wechat.api.dto.qywx.QywxNewExternalUseridDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

@Service
public class ExternalClerkRelatedServiceImpl implements ExternalClerkRelatedService {

    private static final Logger logger = getLogger(ExternalClerkRelatedServiceImpl.class);

    @Autowired
    private TabHaobanExternalClerkRelatedMapper mapper;
    @Autowired
    private ESDataDynamicOperationApiService esDataDynamicOperationApiService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private MemberApiService memberApiService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private TabHaobanStaffClerkRelationMapper staffClerkRelationMapper;
    @Autowired
    private WxUserAddLogService wxUserAddLogService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService ;
    @Autowired
    private MemberUnionidRelatedMapper memberUnionidRelatedMapper ;
    @Autowired
    private QywxUserApiService qywxUserApiService ;
    @Autowired
    private Config config;
    @Autowired
    private TabHaobanRepairExternalUserMapper tabHaobanRepairExternalUserMapper ;
    @Autowired
    private SecretSettingService secretSettingService ;
    
    @Override
    public void insert(TabHaobanExternalClerkRelated related) {
        String clerkId = related.getClerkId();
        String externalUserId = related.getExternalUserId();
        TabHaobanExternalClerkRelated tab = mapper.selectByClerkIdAndExternalUserId(clerkId, externalUserId);
        if (tab == null) {
            String uuId = StringUtil.randomUUID();
            related.setStatusFlag(related.getStatusFlag() != null ? related.getStatusFlag() : 1);
            related.setExternalClerkRelatedId(uuId);
            related.setCreateTime(new Date());
            related.setUpdateTime(new Date());
            mapper.insertSelective(related);
        }
    }

    @Override
    public void delByUserIdAndExternalUserId(MemberUnionidRelatedDTO dto , String wxUserId) {
        int count = mapper.delByUserIdAndExternalUserId(EntityUtil.changeEntityByJSON(TabHaobanExternalClerkRelated.class, dto));
        if (count <= 0) {
            logger.error("无需删除的好友数据：{}", JSON.toJSONString(dto));
            return;
        }
        String staffId = dto.getStaffId() ;
        String externalUserid = dto.getExternalUserid();
        List<TabHaobanExternalClerkRelated> tabHaobanExternalClerkRelatedList = mapper.listByExTernalUseridAndWxUserId(externalUserid, staffId, 2);
        if (CollectionUtils.isEmpty(tabHaobanExternalClerkRelatedList)) {
            logger.info("无解除的好友关系：externalUserid:{},wxUserId:{}", externalUserid, wxUserId);
            return;
        }
        TabHaobanExternalClerkRelated related = tabHaobanExternalClerkRelatedList.get(0);
        String wxEnterpriseId = related.getWxEnterpriseId();
        TabHaobanStaff tabHaobanStaff = staffMapper.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);
        if (null == tabHaobanStaff) {
            logger.error("找不到员工:wxUserId:{}", wxUserId);
            return;
        }

        staffId = tabHaobanStaff.getStaffId();
        List<TabHaobanStaffClerkRelation> tabHaobanStaffClerkRelations = staffClerkRelationMapper.lisByStaffId(wxEnterpriseId, staffId,null);
        if (CollectionUtils.isEmpty(tabHaobanStaffClerkRelations)) {
            logger.error("找不到员工导购关联关系:staffId:{}，wxEnterpriseId：{}", staffId, wxEnterpriseId);
            return;
        }
        String enterpriseId = tabHaobanStaffClerkRelations.get(0).getEnterpriseId();
        //该企微该会员所有关联关系解除后通知会员
        List<TabHaobanExternalClerkRelated> list = mapper.listByExternalUseridAndWxEnterpriseId(dto.getExternalUserid(), wxEnterpriseId, Arrays.asList("1", "3", "4"));
        if (CollectionUtils.isEmpty(list)) {
            //删除的好友关系
            List<TabHaobanExternalClerkRelated> haobanExternalClerkRelatedList = mapper.listByExternalUseridAndWxEnterpriseId(dto.getExternalUserid(), wxEnterpriseId, Collections.singletonList("2"));
            for (TabHaobanExternalClerkRelated externalClerkRelated : haobanExternalClerkRelatedList) {
                String memberId = externalClerkRelated.getMemberId();
                logger.info("通知会员 删除会员后更新为非企微好友 memberId:{}", memberId);
                memberApiService.updateMemberQywxFlag(enterpriseId, memberId, 0);
                memberApiService.updateMemberQywxEveryOccasion(enterpriseId, memberId, 0, new Date());
            }
        } else {
            //删除后 每次回调会员
            List<TabHaobanExternalClerkRelated> relatedList = mapper.listByExTernalUseridAndWxUserId(dto.getExternalUserid(), wxUserId, 2);
            for (TabHaobanExternalClerkRelated externalClerkRelated : relatedList) {
                String memberId = externalClerkRelated.getMemberId();
                logger.info("通知会员 删除会员后更新为非企微好友时间 memberId:{}", memberId);
                memberApiService.updateMemberQywxEveryOccasion(enterpriseId, memberId, 1, new Date());
            }
        }

        //添加日志
        WxUserAddLogQDTO wxUserAddLogQDTO = new WxUserAddLogQDTO();
        wxUserAddLogQDTO.setWxEnterpriseId(wxEnterpriseId);
        wxUserAddLogQDTO.setEnterpriseId(enterpriseId);
        wxUserAddLogQDTO.setMemberId(related.getMemberId());
        wxUserAddLogQDTO.setUnionid(related.getUnionid());
        wxUserAddLogQDTO.setDelFlag(true);
        wxUserAddLogQDTO.setExternalUserid(related.getExternalUserId());
        wxUserAddLogQDTO.setStaffId(related.getStaffId());
        wxUserAddLogQDTO.setClerkId(related.getClerkId());
        wxUserAddLogService.save(wxUserAddLogQDTO);
    }

    @Override
    public void delByUserIdAndExternalUserIdBatch(String wxEnterpriseId, String staffId, List<String> externalUserids) {
        mapper.delByUserIdAndExternalUserIdBatch(staffId, wxEnterpriseId, externalUserids);
    }

    @Override
    public void delByClerkIdAndStaffId(String clerkId, String staffId) {
        mapper.delByClerkIdAndStaffId(clerkId, staffId);
    }

    @Override
    public void delByStoreIdAndStaffId(String storeId, String staffId) {
        mapper.delByStoreIdAndStaffId(storeId, staffId);
    }

    @Override
    public void delByStoreIds(Collection<String> storeIds) {
        mapper.delByStoreIds(storeIds);
    }

    @Override
    public void cleanStaffRelated(String wxEnterpriseId, List<String> staffIds) {
        if (staffIds != null && !staffIds.isEmpty()) {
            mapper.cleanStaffRelated(wxEnterpriseId, staffIds);
        }

    }

    @Override
    public void update(TabHaobanExternalClerkRelated exsitDTO) {
        exsitDTO.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(exsitDTO);

    }

    @Override
    public TabHaobanExternalClerkRelated getByParams(String staffId, String wxEnterpriseId, String name,
                                                     String createTime) {
        return mapper.getByParams(staffId, wxEnterpriseId, name, createTime);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listByExTernalUseridAndWxUserId(String externalUserid, String staffId) {
        return mapper.listByExTernalUseridAndWxUserId(externalUserid, staffId, 1);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listByExTernalUseridAndWxUserIdAllStatus(String wxEnterpriseId, String externalUserid, String staffId) {
        return mapper.listByExTernalUseridAndWxUserIdAllStatus(wxEnterpriseId, externalUserid, staffId);
    }

    @Override
    public ExternalClerkRelatedDTO getByMemberId(String staffId, String memberId) {
        return EntityUtil.changeEntityByJSON(ExternalClerkRelatedDTO.class, mapper.getByMemberId(staffId, memberId));
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listExternalByUnionId(String enterpriseId, String unionId) {
        return mapper.listExternalByUnionId(enterpriseId, unionId);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listExternalByExternalUserid(String wxEnterpriseId, String externalUserid) {
        return mapper.listByExternalUserId(wxEnterpriseId, externalUserid);
    }

    @Override
    public String checkAndUpdateMemberByUnonId(String enterpriseId, String memberId, String unionId) {
        String returnFields = "id";
        try {
            JSONObject searchJson = new JSONObject();
            searchJson.put("id", memberId);
            DynamicSearchDTO search = new DynamicSearchDTO();
            search.setSearchJson(searchJson);
            search.setEnterpriseId(enterpriseId);
            search.setColumnCategoryCode("member");
            if (StringUtils.isNotBlank(returnFields)) {
                search.setReturnFileds(returnFields);
            }
            ESResponseQuerySingle single = esDataDynamicOperationApiService.queryDataSingle(search);
            JSONObject res = single.getRes();
            logger.info("返回数据：{}", JSONObject.toJSONString(res));
            if (res != null && res.size() > 0) {
                return memberId;
            }
            MemberDTO memberDTO = memberService.getMemberByUnionid(unionId, enterpriseId);
            if (null == memberDTO || memberDTO.getMemberId() == null) {
                logger.info("会员不存在：{}", unionId);
                return null;
            }
            //更新会员id
            mapper.updateMemberIdByMemberId(memberDTO.getMemberId(), memberId, enterpriseId);
            return memberDTO.getMemberId();
        } catch (Exception e) {
            logger.info("从搜索引擎中搜索会员数据异常, memberId : {}", memberId);
        }
        return null;
    }
    
    @Override
    public void repairExternalUserId(String wxEnterpriseIds) {
        DingUtils.send("外部联系人修复消息,企业="+wxEnterpriseIds+",tranceid="+ProviderLocalTag.tag.get(), false);
        if(StringUtils.isEmpty(wxEnterpriseIds)) {
            return ;
        }
        Object cache = RedisUtil.getCache("repairExternalUserId") ;
        String[] arr = wxEnterpriseIds.split(",") ;
        for(int i=0;i<arr.length;i++) {
            String wxEnterpriseId = arr[i] ;
            if(StringUtils.isEmpty(wxEnterpriseId)) {
               continue ;
            }
            WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId) ;
            if(null == qwDTO) {
                logger.info("外部联系人检查,企业不存在,wxEnterpriseId={}",wxEnterpriseId);
                continue ;
            }
            SecretSettingDTO secret = this.secretSettingService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.CUSTOMIZED_APP.getVal()) ;
            int pageSize = 800 ;
            int pageNum = 0 ;
            while(true) {
                List<TabHaobanExternalClerkRelated>  list = this.mapper.listPageByWxenterpriseId(wxEnterpriseId, pageNum*pageSize, pageSize) ;
                if(CollectionUtils.isEmpty(list)) {
                    break ;
                }
                pageNum++;
                Set<String> externalUserIdSet = list.stream().map(dto->dto.getExternalUserId()).collect(Collectors.toSet()) ;
                List<String> externalUserIdList = new ArrayList<>();
                externalUserIdList.addAll(externalUserIdSet) ;
                List<MemberUnionidRelated>  unionidlist = this.memberUnionidRelatedMapper.listByWxEnterpriseIdAndExternalUserId(wxEnterpriseId, externalUserIdList) ;
                Map<String,String> map = unionidlist.stream().collect(Collectors.toMap(MemberUnionidRelated::getExternalUserid, MemberUnionidRelated::getSelfExternalUserid,(k1,k2)->k1)) ;
                logger.info("外部联系人检查,第{}次,好友数={},unionid表数={}",pageNum,externalUserIdList.size(),unionidlist.size());
                List<String> delList = new ArrayList<>() ;
                Set<String> checkSet = unionidlist.stream().map(dto->dto.getExternalUserid()).collect(Collectors.toSet()) ;
                List<String> checkList = new ArrayList<>();
                checkList.addAll(checkSet) ;
                for(String item : externalUserIdList) {
                    if(!checkList.contains(item)) {
                        delList.add(item) ;
                    }
                }
                logger.info("外部联系人检查,unionid表不存在记录的数量={},待检查数={}",delList.size(),checkList.size());
                List<String> notExistQwList = new ArrayList<>();
                List<TabHaobanRepairExternalUser> errorList = new ArrayList<>() ;
                if(CollectionUtils.isNotEmpty(checkList)) {
                    List<String> qwList = new ArrayList<>();
                    List<QywxNewExternalUseridDTO> newExternalUseridList = null ;
                    if(null != cache) {
                        newExternalUseridList =  this.qywxUserApiService.getNewExternalUseridByList(qwDTO.getThirdCorpid(), config.getWxSuiteid(), checkList) ;
                    }else {
                        newExternalUseridList =  this.qywxUserApiService.getSelfNewExternalUseridByList(qwDTO.getDkCorpid(), secret.getSecretVal(), checkList) ;
                    }
                    if (null != newExternalUseridList) {
                        logger.info("外部联系人检查,查询企微数={},返回数={}",checkList.size(),newExternalUseridList.size());
                        for (QywxNewExternalUseridDTO qywxNewExternalUseridDTO : newExternalUseridList) {
                            String new_external_userid = qywxNewExternalUseridDTO.getNew_external_userid();
                            String external_userid = qywxNewExternalUseridDTO.getExternal_userid();
                            qwList.add(external_userid) ;
                            if(new_external_userid.equals(external_userid)) {
                                continue ;
                            }else {
                                logger.info("外部联系人检查,数据异常={}",new_external_userid,external_userid);
                                TabHaobanRepairExternalUser user = new TabHaobanRepairExternalUser();
                                user.setId(UniqueIdUtils.uniqueLong());
                                user.setUserId(external_userid);
                                user.setWxEnterpriseId(wxEnterpriseId);
                                user.setStatusFlag(111);
                                user.setRemark(new_external_userid);
                                errorList.add(user) ;
                            }
                        }
                        if(checkList.size()!=newExternalUseridList.size()) {
                            for(String item : checkList) {
                                if(!qwList.contains(item)) {
                                    notExistQwList.add(item) ;
                                }
                            }
                        }
                    }else {
                        logger.info("外部联系人检查,调用企业异常");
                    }
                    // 通过外部联系人查询不到
                    if(CollectionUtils.isNotEmpty(notExistQwList)) {
                        // 需要用自建的查询的
                        List<String> selfList = new ArrayList<>();
                        List<TabHaobanRepairExternalUser> addList = new ArrayList<>();
                        for(String item : notExistQwList) {
                            TabHaobanRepairExternalUser user = new TabHaobanRepairExternalUser();
                            user.setId(UniqueIdUtils.uniqueLong());
                            user.setUserId(item);
                            if(StringUtils.isNotBlank(map.get(item)) && !map.get(item).equals(item)) {
                                selfList.add(map.get(item)) ;
                            }
                            user.setSelfUserId(map.get(item));
                            user.setWxEnterpriseId(wxEnterpriseId);
                            user.setStatusFlag(11);
                            user.setRemark("企微查询不到");
                            addList.add(user) ;
                        }
                        
                        if(CollectionUtils.isNotEmpty(selfList)) {
                            newExternalUseridList = null ;
                            if(null != cache) {
                                newExternalUseridList =  this.qywxUserApiService.getNewExternalUseridByList(qwDTO.getThirdCorpid(), config.getWxSuiteid(), selfList) ;
                            }else {
                                newExternalUseridList =  this.qywxUserApiService.getSelfNewExternalUseridByList(qwDTO.getDkCorpid(), secret.getSecretVal(), selfList) ;
                            }
                            Map<String,String> selfMap = new HashMap<>() ;
                            if(null != newExternalUseridList) {
                                selfMap = newExternalUseridList.stream().collect(Collectors.toMap(QywxNewExternalUseridDTO::getExternal_userid, QywxNewExternalUseridDTO::getNew_external_userid,(k1,k2)->k1)) ;
                            }
                            for(TabHaobanRepairExternalUser item : addList) {
                                item.setNewUserId(selfMap.get(item.getSelfUserId()));
                            }
                        }
                        this.tabHaobanRepairExternalUserMapper.batchInsert(addList) ;
                    }
                    // unionid表不存在的记录
                    if(CollectionUtils.isNotEmpty(delList)) {
                        List<TabHaobanRepairExternalUser> addList = new ArrayList<>();
                        for(String item : delList) {
                            TabHaobanRepairExternalUser user = new TabHaobanRepairExternalUser();
                            user.setId(UniqueIdUtils.uniqueLong());
                            user.setUserId(item);
                            user.setWxEnterpriseId(wxEnterpriseId);
                            user.setStatusFlag(1);
                            user.setRemark("unionid不存在记录");
                            addList.add(user) ;
                        }
                        this.tabHaobanRepairExternalUserMapper.batchInsert(addList) ;
                    }
                    
                    // 错误的ID
                    if(CollectionUtils.isNotEmpty(errorList)) {
                        this.tabHaobanRepairExternalUserMapper.batchInsert(errorList) ;
                    }
                }
            }
        }
    }
}
